from ..config.config import BASE_SYSTEM_TEXT
import random
from typing import Union, Any, Coroutine
from abc import abstractmethod, ABC

class History:
    def __init__(self, system_prompt: str = BASE_SYSTEM_TEXT):
        self.history: list[dict[str, str]] = []
        if system_prompt is not None:
            self.add_to_history("system", system_prompt)

    def add_to_history(self, role: str, content: str):
        self.history.append({
            "role": role,
            "content": content
        })


class ParentModel(ABC):
    def __init__(self, api_key, history: History = History(), 
                 models_list: list[str] = ["openai/gpt-oss-120b",  "llama-3.3-70b-versatile",  "openai/gpt-oss-safeguard-20b"],
                 use_random: bool = True):
        self.history = history
        self.api_keys = api_key

        self.current_model_index = 0
        self.current_key_index = 0

        
        models_list = models_list.copy()
        if use_random:
            random.shuffle(models_list)
        self.regen_models_name = models_list

    @abstractmethod
    def generate_answer(self, with_history: bool = True, prompt: list[dict[str, str]] | None = None) -> Union[str, Coroutine[Any, Any, str]]:
        return ""
    
    @abstractmethod
    def get_answer_without_history(self, prompt: list[dict[str, str]]) -> Union[str, Coroutine[Any, Any, str]]:
        return ""
    
    @abstractmethod
    def get_answer(self, prompt: str) -> Union[str, Coroutine[Any, Any, str]]:
        return ""



class Model(ParentModel):

    def generate_answer(self, with_history: bool = True, prompt: list[dict[str, str]]  |  None = None) -> str:
        return "answer"
    
    def get_answer_without_history(self, prompt: list[dict[str, str]]) -> str:
        return self.generate_answer(with_history=False, prompt=prompt)

    def get_answer(self, prompt: str) -> str:
        self.history.add_to_history("user", prompt)
        model_answer = self.generate_answer()
        self.history.add_to_history("assistant", model_answer)
        return model_answer
    
class AsyncModel(ParentModel):

    async def generate_answer(self, with_history: bool = True, prompt: list[dict[str, str]] | None = None) -> str:
        return "answer"
    
    async def get_answer_without_history(self, prompt: list[dict[str, str]]) -> str:
        return await self.generate_answer(with_history=False, prompt=prompt)

    async def get_answer(self, prompt: str) -> str:
        self.history.add_to_history("user", prompt)
        model_answer = await self.generate_answer()
        self.history.add_to_history("assistant", model_answer)
        return model_answer





import sys
from autodocgenerator.manage import Manager
from autodocgenerator.factory.base_factory import DocFactory
from autodocgenerator.factory.modules.general_modules import CustomModule, CustomModuleWithOutContext
from autodocgenerator.factory.modules.intro import IntroLinks, IntroText, BaseModule
from autodocgenerator.ui.progress_base import ConsoleGtiHubProgress
from autodocgenerator.auto_runner.config_reader import Config, read_config, StructureSettings
from autodocgenerator.engine.models.gpt_model import GPTModel, AsyncGPTModel, GPT4oModel, Model
from autodocgenerator.engine.models.azure_model import AzureModel

from autodocgenerator.engine.config.config import GOOGLE_EMBEDDING_API_KEY, MODELS_API_KEYS, \
    TYPE_OF_MODEL, OUTPUT_GITHUB_FILE
from autodocgenerator.postprocessor.embedding import Embedding
from autodocgenerator.auto_runner.check_git_status import check_git_status
from autodocgenerator.schema.cache_settings import CheckGitStatusResultSchema

MODELS_CONFIG = {
    "git": GPT4oModel,
    "azure": AzureModel,
    "groq_cloud": GPTModel
}

def gen_doc(project_path: str, 
            config: Config, 
            custom_modules: list[BaseModule], 
            structure_settings: StructureSettings) -> str:
    
    sync_model: Model
    sync_model = MODELS_CONFIG.get(TYPE_OF_MODEL, GPT4oModel)(MODELS_API_KEYS, use_random=False)

    embedding_model = Embedding(GOOGLE_EMBEDDING_API_KEY)
    
    
    manager = Manager(
        project_path, 
        config=config,
        llm_model=sync_model,
        embedding_model=embedding_model,
        progress_bar=ConsoleGtiHubProgress(), 
    )

    change_info: CheckGitStatusResultSchema = check_git_status(manager)
    print(change_info)

    if not change_info.need_to_remake and not change_info.remake_gl_file:
        manager.load_all_info()
        manager.save()
        if OUTPUT_GITHUB_FILE:
            with open(OUTPUT_GITHUB_FILE, "a") as f:
                f.write("skip_next=true\n")
        print("Stopping workflow early")
        sys.exit(0)
        return manager.doc_info.doc.get_full_doc()
    

    
    manager.generate_code_file()
    if structure_settings.use_global_file:
        manager.generate_global_info(compress_power=4, is_reusable=not change_info.remake_gl_file)
    
    manager.generete_doc_parts(max_symbols=structure_settings.max_doc_part_size, with_global_file=structure_settings.use_global_file)
   

    manager.factory_generate_doc(DocFactory(*custom_modules))
    if structure_settings.include_order:
        manager.order_doc()
    
    additionals_modules: list[BaseModule] = []

    if structure_settings.include_intro_text:
        additionals_modules.append(IntroText())

    if structure_settings.include_intro_links:
        additionals_modules.append(IntroLinks())

    
    manager.factory_generate_doc(DocFactory(*additionals_modules, with_splited=False), to_start=True)
    manager.create_embedding_layer()
    manager.clear_cache()

    manager.save()
    return manager.doc_info.doc.get_full_doc()

if __name__ == "__main__":
    with open(r"autodocconfig.yml", "r", encoding="utf-8") as file:
        config_data = file.read()
    config, custom_modules, structure_settings = read_config(config_data)

    output_doc = gen_doc(
        r".",
        config,
        custom_modules,
        structure_settings
    )

    






from .preprocessor.spliter import split_data, gen_doc_parts
from .preprocessor.compressor import compress_to_one
from .engine.models.model import Model
import os
from .preprocessor.code_mix import CodeMix
from .factory.base_factory import DocFactory
from .ui.progress_base import BaseProgress
from .ui.logging import BaseLogger, InfoLog, ErrorLog, WarningLog, FileLoggerTemplate
from .postprocessor.sorting import get_order, split_text_by_anchors
from .config.config import Config
from .schema.doc_schema import DocContent, DocHeadSchema, DocInfoSchema
from .schema.cache_settings import CacheSettings, CheckGitStatusResultSchema
from .postprocessor.embedding import Embedding
from .preprocessor.checker import have_to_change

import json

class Manager:
    CACHE_FOLDER_NAME = ".auto_doc_cache"

    FILE_NAMES = {
        "code_mix": "code_mix.txt",
        "global_info": "global_info.md",
        "logs": "report.txt",
        "output_doc": "output_doc.md",
        "info": "info.json",
        ".auto_doc_cache_file": ".auto_doc_cache_file.json"
    }


    def __init__(self, project_directory: str, 
                 config: Config, 
                 llm_model: Model,
                 embedding_model: Embedding,
                 progress_bar: BaseProgress = BaseProgress()):
        
    
        self.doc_info = DocInfoSchema()
        self.config = config

        self.project_directory = project_directory
        self.progress_bar = progress_bar

        self.llm_model = llm_model
        self.embedding_model = embedding_model

        self.logger = BaseLogger()
        self.logger.set_logger(FileLoggerTemplate(self.get_file_path("logs"), log_level=self.config.pbc.log_level))

        self.init_folder_system(self.project_directory)

    
    def init_folder_system(self, project_directory):
        cache_path = os.path.join(project_directory, self.CACHE_FOLDER_NAME)

        if not os.path.isdir(cache_path):
            os.mkdir(cache_path)

        cache_file_path = self.get_file_path(".auto_doc_cache_file", is_outside=True)
        if not os.path.isfile(cache_file_path):
            with open(cache_file_path, "w", encoding="utf-8") as file:
                file.write(CacheSettings().model_dump_json())

        self.cache_settings = CacheSettings.model_validate_json(self.read_file_by_file_key(".auto_doc_cache_file", is_outside=True))
        
    def read_file_by_file_key(self, file_key: str, is_outside: bool = False):
        try:
            with open(self.get_file_path(file_key, is_outside), "r", encoding="utf-8") as file:
                data = file.read()
        except:
            data = None
        return data

    def get_file_path(self, file_key: str, is_outside: bool = False):
        if is_outside:
            return os.path.join(self.project_directory, self.FILE_NAMES[file_key]) 
        return os.path.join(self.project_directory, self.CACHE_FOLDER_NAME, self.FILE_NAMES[file_key]) 

    def generate_code_file(self):
        self.logger.log(InfoLog("Starting code mix generation..."))
        cm = CodeMix(self.project_directory, self.config.ignore_files)
        code_mix = cm.build_repo_content()

        self.logger.log(InfoLog("Code mix generation completed."))
        self.doc_info.code_mix = code_mix
        self.progress_bar.update_task()

    def generate_global_info(self, compress_power: int = 4, max_symbols: int = 10000, is_reusable: bool = False):
        if is_reusable:
            if self.cache_settings.doc.global_info != "":
                self.doc_info.global_info = self.cache_settings.doc.global_info
                self.progress_bar.update_task()

                return

        full_code_mix = self.doc_info.code_mix
        data = split_data(full_code_mix, max_symbols)

        global_result = compress_to_one(data, self.llm_model, self.config.get_project_settings(), compress_power=compress_power, progress_bar=self.progress_bar)
        self.doc_info.global_info = global_result

        self.progress_bar.update_task()

    def generete_doc_parts(self, max_symbols=5_000, with_global_file: bool = False):
        full_code_mix = self.doc_info.code_mix
        global_file = self.doc_info.global_info if with_global_file else None

        self.logger.log(InfoLog("Starting synchronous documentation generation by parts..."))
        result = gen_doc_parts(full_code_mix,
                                max_symbols, self.llm_model, self.config.get_project_settings(),
                                self.config.language, self.progress_bar, global_info=global_file)
        
        result = split_text_by_anchors(result)
        
        for el in result:
            self.doc_info.doc.add_parts(el, DocContent(content=result[el]))

        self.logger.log(InfoLog("Documentation generation by parts completed."))
        self.progress_bar.update_task()

    def factory_generate_doc(self, doc_factory: DocFactory, to_start: bool = False): 
        curr_doc = self.doc_info.doc.get_full_doc()
        code_mix = self.doc_info.code_mix
        global_info = self.doc_info.global_info


        info = {
            "language": self.config.language,
            "full_data": curr_doc,
            "code_mix": code_mix,
            "global_info": global_info
        }

        self.logger.log(InfoLog(f"""Starting factory documentation generation \n
                                {" ".join([type(module).__name__ for module in doc_factory.modules])} \n
                                Input params: {" ".join([f"{key}" for key, value in info.items()])}
                                """))

        result = doc_factory.generate_doc(info, self.llm_model, self.progress_bar)
        self.logger.log(InfoLog("Factory documentation generation completed."))
        if to_start:
            self.doc_info.doc = result + self.doc_info.doc
        else:
            self.doc_info.doc = self.doc_info.doc + result

        self.progress_bar.update_task()

    def check_sense_changes(self, changes: list[dict[str, str]]) -> CheckGitStatusResultSchema:
        result = have_to_change(self.llm_model, changes, self.cache_settings.doc.global_info)
        return result

    def create_embedding_layer(self) -> None:
        for el in self.doc_info.doc.parts:
            self.doc_info.doc.parts[el].init_embedding(self.embedding_model)

    def order_doc(self):
        result = get_order(self.llm_model, self.doc_info.doc.content_orders)
        self.doc_info.doc.content_orders = result

    def clear_cache(self):
        if not self.config.pbc.save_logs:
            os.remove(self.get_file_path("logs"))
    
    def load_all_info(self):
        self.doc_info = DocInfoSchema.model_validate_json(self.read_file_by_file_key(".auto_doc_cache_file", is_outside=True))

    def save(self) -> None:
        with open(self.get_file_path("output_doc"), "w", encoding="utf-8") as file:
            file.write(self.doc_info.doc.get_full_doc())

        self.cache_settings.doc = self.doc_info
        with open(self.get_file_path(".auto_doc_cache_file", is_outside=True), "w", encoding="utf-8") as file:
            file.write(self.cache_settings.model_dump_json())


import re
from ..engine.models.model import Model
from ..ui.logging import BaseLogger, InfoLog, WarningLog, ErrorLog

def extract_links_from_start(chunks) -> tuple[list[str], bool]:
    links = []
    pattern = r'^<a name=["\']?(.*?)["\']?></a>'
    
    have_to_del_first = False
    for chunk in chunks:
        is_find = False
        match = re.search(pattern, chunk.strip())
        if match:
            anchor_name = match.group(1)
            if len(anchor_name) > 5:
                is_find = True
                links.append("#" + anchor_name)
        if not is_find:
            have_to_del_first = True
                
    return links, have_to_del_first


def split_text_by_anchors(text: str) -> dict[str, str]:
    pattern = r'(?=<a name=["\']?[^"\'>\s]{6,200}["\']?></a>)'
    chunks = re.split(pattern, text)
    result_chanks = [chunk.strip() for chunk in chunks if chunk.strip()]
    all_links, have_to_del_first = extract_links_from_start(result_chanks)


    start_link_index = text.find("<a name")
    if start_link_index > 10 or have_to_del_first:
        result_chanks.pop(0)
    result = {}


    if len(all_links) != len(result_chanks):
        raise Exception("Somthing with anchors")

    for i in range(len(all_links)):
        result[all_links[i]] = result_chanks[i]

    return result


def get_order(model: Model, chanks: list[str]) -> list:
    logger = BaseLogger()
    logger.log(InfoLog("Start ordering"))
    logger.log(InfoLog(f"chanks name: {chanks}", level=1))


    prompt = [ #TODO tranport promt to prompts
        {
            "role": "user",
            "content": f"""Sort the following titles semantically (group related topics together). 
                        Return ONLY a comma-separated list of the sorted titles. 
                        Do not include any introductory text, explanations, or concluding remarks.
                        leave # in title.
                        do not skip any title
                        Titles:
                        {chanks}
            """
        }
    ]
    result = model.get_answer_without_history(prompt)
    new_result = list(map(lambda x: x.strip(), result.split(",")))
    logger.log(InfoLog(f"End ordering result list {new_result}"))

    
        
    return new_result


if __name__ == "__main__":
    with open(r"C:\Users\huina\Python Projects\Impotant projects\AutoDocGenerateGimini\README.md", "r", encoding="utf-8") as file:
        data = file.read()

    print(split_text_by_anchors(data))





from google import genai
from google.genai import types
import numpy as np
from typing import Any


def bubble_sort_by_dist(arr: list) -> list:
    n = len(arr)
    for i in range(n):
        for j in range(0, n - i - 1):
            if arr[j][1] > arr[j + 1][1]:
                arr[j], arr[j + 1] = arr[j + 1], arr[j]
    return arr

def get_len_btw_vectors(vector1, vector2) -> float:
    diff = np.linalg.norm(vector1 - vector2)
    return float(diff)

def sort_vectors(root_vector, other: dict[str, Any]) -> list[str]:
    sort_list: list[tuple[str, float]] = []

    for el in other:
        len_btw = get_len_btw_vectors(root_vector, other[el])
        sort_list.append((el, len_btw))

    sort_list = bubble_sort_by_dist(sort_list)
    result_list = [el[0] for el in sort_list]

    return result_list

class Embedding:
    def __init__(self, api_key: str):
        self.client = genai.Client(api_key=api_key)

    def get_vector(self, prompt: str) -> list:
        text_response = self.client.models.embed_content(
            model='gemini-embedding-2-preview',
            contents=prompt,
            config=types.EmbedContentConfig(output_dimensionality=768)
        )

        if text_response.embeddings is None:
            raise Exception("promblem with embedding")

        return list(text_response.embeddings[0])[0][1]
    





from abc import ABC, abstractmethod
from ..engine.models.model import Model, AsyncModel
from ..ui.progress_base import BaseProgress
from ..ui.logging import BaseLogger, InfoLog, ErrorLog, WarningLog
from ..schema.doc_schema import DocContent, DocHeadSchema
from ..postprocessor.sorting import split_text_by_anchors

class BaseModule(ABC):
    def __init__(self):
        pass
    
    @abstractmethod
    def generate(self, info: dict, model: Model):
        ...


class DocFactory:
    def __init__(self, *modules, with_splited: bool = True):
        self.modules: list[BaseModule] = modules # type: ignore
        self.logger = BaseLogger()
        self.with_splited = with_splited

    def generate_doc(self, info: dict, model: Model, progress: BaseProgress) -> DocHeadSchema:
        doc_head = DocHeadSchema()
        progress.create_new_subtask("Generate parts", len(self.modules))
        for i, module in enumerate(self.modules):
            module_result = module.generate(info, model)
            if self.with_splited:
                splited_result = split_text_by_anchors(module_result)
                for el in splited_result:
                    doc_head.add_parts(el, DocContent(content=splited_result[el]))
            else:
                task_name = f"{module.__class__.__name__}_{i}"
                doc_head.add_parts(task_name, DocContent(content=module_result))

            self.logger.log(InfoLog(f"Module {module.__class__.__name__} generated its part of the documentation."))
            self.logger.log(InfoLog(f"Module Output: {module_result}", level=2))
            progress.update_task()
        progress.remove_subtask()

        return doc_head




from pydantic import BaseModel, Field
from .doc_schema import DocInfoSchema

class CacheSettings(BaseModel):
    last_commit: str = ""
    doc: DocInfoSchema = Field(default_factory=DocInfoSchema)


class CheckGitStatusResultSchema(BaseModel):
    need_to_remake: bool
    remake_gl_file: bool



def _print_welcome():
    BLUE = "\033[94m"
    BOLD = "\033[1m"
    CYAN = "\033[96m"
    RESET = "\033[0m"

    ascii_logo = f"""
    {BLUE}{BOLD}    ___     ____     ______
       /   |   / __ \   / ____/
      / /| |  / / / /  / / __  
     / ___ | / /_/ /  / /_/ /  
    /_/  |_|/_____/   \____/   
    {RESET}"""
    
    print(ascii_logo)
    print(f"{CYAN}ADG Library{RESET} | {BOLD}Status:{RESET} Ready to work V0.0.6")
    print(f"{'—' * 35}\n")

_print_welcome()


from .ui.logging import BaseLogger, BaseLoggerTemplate, InfoLog, ErrorLog, WarningLog

logger = BaseLogger()
logger.set_logger(BaseLoggerTemplate())


import os
from pathlib import Path
import fnmatch
from ..ui.logging import InfoLog, BaseLogger

class CodeMix:
    def __init__(self, root_dir=".", ignore_patterns=None):
        self.root_dir = Path(root_dir).resolve()
        self.ignore_patterns = ignore_patterns or []
        self.logger = BaseLogger()

    def should_ignore(self, path: str) -> bool:
        relative_path = path.relative_to(self.root_dir) # type: ignore
        path_str = str(relative_path)
        
        for pattern in self.ignore_patterns:
            if fnmatch.fnmatch(path_str, pattern) or \
               fnmatch.fnmatch(os.path.basename(path_str), pattern) or \
               any(fnmatch.fnmatch(p, pattern) for p in path.parts): # type: ignore
                return True
        return False

    def build_repo_content(self):
        content = []
        content.append("Repository Structure:")
        
        for path in sorted(self.root_dir.rglob("*")):
            if self.should_ignore(path):
                self.logger.log(InfoLog(f"Ignored: {path}", level=1))
                continue
            
            depth = len(path.relative_to(self.root_dir).parts)
            indent = "  " * (depth - 1)
            name = f"{path.name}/" if path.is_dir() else path.name
            content.append(f"{indent}{name}")
        
        content.append("\n" + "="*20 + "\n")

        for path in sorted(self.root_dir.rglob("*")):
            if path.is_file() and not self.should_ignore(path):
                try:
                    relative_path = path.relative_to(self.root_dir)
                    content.append(f'<file path="{relative_path}">')
                    content.append(path.read_text(encoding="utf-8", errors="ignore"))
                    content.append("</file>\n")
                except Exception as e:
                    content.append(f"Error reading {path}: {e}")

        return "\n".join(content)

ignore_list = [
    "*.pyo", "*.pyd", "*.pdb", "*.pkl", "*.log", "*.sqlite3", "*.db",
    "venv", "env", ".venv", ".env", ".vscode", ".idea", "*.iml", ".gitignore", ".ruff_cache",
    "*.pyc", "__pycache__", ".git", ".coverage", "htmlcov", "migrations", "*.md", "static", "staticfiles", ".mypy_cache"
]


if __name__ == "__main__":
    packer = CodeMix(root_dir=r"C:\Users\huina\Python Projects\Kwork\ClickerProject\ClickerApp", ignore_patterns=ignore_list)
    # packer.build_repo_content("codemix.txt")
    print("Файл успешно создан!")



from .model import Model, AsyncModel,  History
from ..exceptions import ModelExhaustedException
from ...ui.logging import BaseLogger, InfoLog, ErrorLog, WarningLog
from azure.ai.inference import ChatCompletionsClient
from azure.ai.inference.models import SystemMessage, UserMessage, ChatRequestMessage
from azure.core.credentials import AzureKeyCredential
import re


class AzureModel(Model):
    def __init__(self, api_key, history = History(), 
                 models_list: list[str] = ["deepseek/DeepSeek-V3-0324"], 
                 use_random: bool = True):
        super().__init__(api_key, history, models_list, use_random)
        self.client = ChatCompletionsClient(
            endpoint="https://models.github.ai/inference",
            credential=AzureKeyCredential(self.api_keys[self.current_key_index]),
        )
        self.logger = BaseLogger()

    def _clean_deepseek_response(self, text: str) -> str:
        """Удаляет блок <think>...</think> и лишние пробелы."""
        cleaned_text = re.sub(r'<think>.*?</think>', '', text, flags=re.DOTALL)
        return cleaned_text.strip()

    def _parse_prompt(self, data: list[dict[str, str]]) -> list[UserMessage | SystemMessage]:
        result: list[UserMessage | SystemMessage] = []
        for el in data:
            if el["role"] == "system":
                result.append(SystemMessage(el["content"]))
                continue

            if el["role"] == "user":
                result.append(UserMessage(el["content"]))
                continue

        return result


    def generate_answer(self, with_history: bool = True, prompt: list[dict[str, str]] |  None = None) -> str:
        self.logger.log(InfoLog("Generating answer..."))
        if with_history:
            messages = self.history.history
        elif prompt is not None:
            messages = prompt
        
        chat_completion = None
        model_name = None
        
        parse_messages: list[UserMessage | SystemMessage] = self._parse_prompt(messages)
        while True:
            if len(self.regen_models_name) <= 0:
                self.logger.log(ErrorLog("No models available for use."))
                raise ModelExhaustedException("No models available for use.")
            
            model_name = self.regen_models_name[self.current_model_index]
            try:
                response = self.client.complete(
                    messages=parse_messages, #type: ignore
                    temperature=0.2,
                    top_p=1.0,
                    max_tokens=10000,
                    model=model_name
                )

                break
            except Exception as e:
                print(e)
                self.logger.log(WarningLog(f"Model {model_name} failed with error: {str(e)}. Trying next model..."))
                self.current_key_index = 0 if self.current_key_index + 1 >= len(self.api_keys) else self.current_key_index + 1
                if self.current_key_index == 0:
                    self.current_model_index = 0 if self.current_model_index + 1 >= len(self.regen_models_name) else self.current_model_index + 1
                    
                self.client = ChatCompletionsClient(
                    endpoint="https://models.github.ai/inference",
                    credential=AzureKeyCredential(self.api_keys[self.current_key_index]),
                )




        result = self._clean_deepseek_response(response.choices[0].message.content)
        self.logger.log(InfoLog(f"Generated answer with model {model_name}."))
        self.logger.log(InfoLog(f"Answer: {result}", level=2))
        if result is None:
            return ""
        
        return result


from pydantic import BaseModel, Field
import random
from ..postprocessor.embedding import Embedding
from typing import Any

class DocContent(BaseModel):
    content: str
    embedding_vector: list | None = None

    def init_embedding(self, embedding_model: Embedding):
        self.embedding_vector = embedding_model.get_vector(self.content)

class DocHeadSchema(BaseModel):
    content_orders: list[str] = []
    parts: dict[str, DocContent] = {}

    def add_parts(self, name, content: DocContent):
        iter_c = 0
        while name in self.content_orders:
            name = f"{name}_{iter_c}"
            iter_c += 1

        self.content_orders.append(name)
        self.parts[name] = content

    def get_full_doc(self, split_el: str = "\n") -> str:
        output_doc: str = ""
        for el in self.content_orders:
            output_doc += self.parts[el].content + split_el

        return output_doc
    
    def __add__(self, other: "DocHeadSchema") -> "DocHeadSchema":
        if not isinstance(other, DocHeadSchema):
            return NotImplemented
        
        for el in other.content_orders:
            self.add_parts(el, other.parts[el])
        return self

class DocInfoSchema(BaseModel):
    global_info: str = ""
    code_mix: str = ""
    doc: DocHeadSchema = Field(default_factory=DocHeadSchema)




import os
import requests

def main():
    api_key = os.getenv("ADG_API_TOKEN")
    default_server_url = os.getenv("DEFAULT_SERVER_URL")
    
    if not api_key:
        print("Error: ADG_API_TOKEN is not set")
        exit(1)
    
    if not default_server_url:
        print("Error: DEFAULT_SERVER_URL is not set")
        exit(1)

    url = f"{default_server_url}/github/get_api_keys" 
    headers = {"Authorization": f"Bearer {api_key}"}

    try:
        response = requests.get(url, headers=headers)
        response.raise_for_status()
        data = response.json()["data"]

        if data["status"] != "success":
            raise Exception(f"API returned error status: {data.get('message', 'No message provided')}")

        github_token = data.get("github_token")
        google_token = data.get("google_token")

        # Записываем в специальный файл GitHub Env, чтобы следующие шаги видели эти ключи
        env_file = os.getenv('GITHUB_ENV')
        if env_file:
            with open(env_file, "a") as f:
                f.write(f"MODELS_API_KEYS={github_token}\n")
                f.write(f"GOOGLE_EMBEDDING_API_KEY={google_token}\n")
                f.write(f"TYPE_OF_MODEL=git\n") 
            print("Keys successfully written to GITHUB_ENV")
        else:
            print(f"Local result: MODELS_API_KEYS={github_token}, GOOGLE_EMBEDDING_API_KEY={google_token}")

    except Exception as e:
        print(f"An error occurred: {e}")
        exit(1)

if __name__ == "__main__":
    main()


from ..base_factory import BaseModule
from ...engine.models.model import Model
from ...postprocessor.custom_intro import generete_custom_discription, generete_custom_discription_without
from ...preprocessor.spliter import split_data

class CustomModule(BaseModule):

    def __init__(self, discription: str):
        super().__init__()
        self.discription = discription  

    def generate(self, info: dict, model: Model):
        result = generete_custom_discription(split_data(info.get("code_mix"), max_symbols=5000), model, self.discription, info.get("language"))
        return result
    
class CustomModuleWithOutContext(BaseModule):

    def __init__(self, discription: str):
        super().__init__()
        self.discription = discription  

    def generate(self, info: dict, model: Model):
        result = generete_custom_discription_without(model, self.discription, info.get("language"))
        return result


import yaml #type: ignore
from autodocgenerator.factory.modules.general_modules import CustomModule, CustomModuleWithOutContext, BaseModule
from ..config.config import Config, ProjectBuildConfig
from typing import Any


class StructureSettings:
    include_intro_links = True
    include_order = True
    use_global_file = True
    max_doc_part_size = 5_000
    include_intro_text = True

    def load_settings(self, data: dict[str, Any]):
        for key, el in data.items():
            setattr(self, key, el)


def read_config(file_data: str) -> tuple[Config, list[BaseModule], StructureSettings]:
    data = yaml.safe_load(file_data)
    config : Config = Config()

    ignore_files = data.get("ignore_files", [])
    language = data.get("language", "en")

    project_name = data.get("project_name")
    project_additional_info = data.get("project_additional_info", {})

    project_settings = data.get("build_settings", {})
    pcs = ProjectBuildConfig()
    pcs.load_settings(project_settings)
    
    config.set_language(language).set_project_name(project_name).set_pcs(pcs)

    for pattern in ignore_files:
        config.add_ignore_file(pattern)

    for key in project_additional_info:
        config.add_project_additional_info(key, project_additional_info[key])

    custom_discriptions = data.get("custom_descriptions", [])

    custom_modules: list[BaseModule] = [CustomModuleWithOutContext(custom_discription[1:])  if custom_discription[0] == "%" else CustomModule(custom_discription) for custom_discription in custom_discriptions]

    structure_settings = data.get("structure_settings", {})
    structure_settings_object = StructureSettings()
    structure_settings_object.load_settings(structure_settings)

    return config, custom_modules, structure_settings_object